package software.eskimo.capacitor.sockets;

import android.os.Handler;
import android.os.Looper;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.Socket;
import java.net.InetSocketAddress;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import javax.net.ssl.SSLSocket;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManager;
import javax.net.ssl.X509TrustManager;
import javax.net.ssl.SSLContext;

interface SocketDelegate {
    void didChangeState(String socketId, String state);
    void didReceiveMessage(String socketId, String message);
}

public class SocketConnection {

    public String id;
    private String host;
    private int port;
    private boolean useTLS;
    private boolean acceptInvalidCertificates;
    private SocketDelegate delegate;

    private Socket socket;
    private ExecutorService executor;

    public SocketConnection(String id, String host, int port, boolean useTLS, boolean acceptInvalidCertificates) {
        this.id = id;
        this.host = host;
        this.port = port;
        this.useTLS = useTLS;
        this.acceptInvalidCertificates = acceptInvalidCertificates;
        this.executor = Executors.newSingleThreadExecutor();
    }

    public void setDelegate(SocketDelegate delegate) {
        this.delegate = delegate;
    }

    public void connect() {
        executor.execute(() -> {
            try {
                if (useTLS) {
                    SSLSocketFactory factory;
                    if (acceptInvalidCertificates) {
                        SSLContext sslContext = SSLContext.getInstance("TLS");
                        TrustManager[] trustManagers = new TrustManager[]{
                            new X509TrustManager() {
                                @Override
                                public java.security.cert.X509Certificate[] getAcceptedIssuers() {
                                    return null;
                                }

                                @Override
                                public void checkClientTrusted(java.security.cert.X509Certificate[] certs, String authType) {
                                }

                                @Override
                                public void checkServerTrusted(java.security.cert.X509Certificate[] certs, String authType) {
                                }
                            }
                        };
                        sslContext.init(null, trustManagers, new java.security.SecureRandom());
                        factory = sslContext.getSocketFactory();
                    } else {
                        factory = (SSLSocketFactory) SSLSocketFactory.getDefault();
                    }
                    socket = factory.createSocket();
                    socket.connect(new InetSocketAddress(host, port));
                    delegate.didChangeState(id, "connected");
                } else {
                    socket = new Socket();
                    socket.connect(new InetSocketAddress(host, port));
                    delegate.didChangeState(id, "connected");
                }

                listenForMessages();
            } catch (Exception e) {
                e.printStackTrace();
                delegate.didChangeState(id, "disconnected");
            }
        });
    }

    public void send(String message) {
        executor.execute(() -> {
            try {
                if (socket != null && socket.isConnected()) {
                    OutputStream outputStream = socket.getOutputStream();
                    outputStream.write(message.getBytes());
                    outputStream.flush();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });
    }

    public void disconnect() {
        try {
            if (socket != null && !socket.isClosed()) {
                socket.close();
                delegate.didChangeState(id, "disconnected");
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void listenForMessages() {
        executor.execute(() -> {
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                char[] buffer = new char[1024]; // Adjust buffer size as needed
                int charsRead;
                
                while ((charsRead = reader.read(buffer)) != -1) {
                    String message = new String(buffer, 0, charsRead);
                    delegate.didReceiveMessage(id, message);
                }
            } catch (Exception e) {
                e.printStackTrace();
                disconnect();
            }
        });
    }

}
