package software.eskimo.capacitor.sockets;

import android.util.Log;
import com.getcapacitor.JSObject;
import com.getcapacitor.Plugin;
import com.getcapacitor.PluginCall;
import com.getcapacitor.PluginMethod;
import com.getcapacitor.annotation.CapacitorPlugin;

@CapacitorPlugin(name = "Sockets")
public class SocketsPlugin extends Plugin {

    private Sockets implementation;

    @Override
    public void load() {
        implementation = new Sockets(this);
    }

    public void notifyStateChange(String socketId, String state) {
        JSObject data = new JSObject();
        data.put("id", socketId);
        data.put("state", state);
        notifyListeners("state", data);
    }

    public void notifyMessageReceived(String socketId, String message) {
        JSObject data = new JSObject();
        data.put("id", socketId);
        data.put("message", message);

        Log.e("SocketsPlugin", "Received message: " + message);
       
        notifyListeners("message", data);
    }

    @PluginMethod
    public void create(PluginCall call) {
        String id = call.getString("id", java.util.UUID.randomUUID().toString());
        String host = call.getString("host");
        int port = call.getInt("port");
        boolean useTLS = call.getBoolean("useTLS", false);
        boolean acceptInvalidCertificates = call.getBoolean("acceptInvalidCertificates", false);

        implementation.create(id, host, port, useTLS, acceptInvalidCertificates);
        call.resolve();
    }

    @PluginMethod
    public void connect(PluginCall call) {
        String id = call.getString("id");
        implementation.connect(id);
        call.resolve();
    }

    @PluginMethod
    public void send(PluginCall call) {
        String id = call.getString("id");
        String message = call.getString("message");
        implementation.send(id, message);
        call.resolve();
    }

    @PluginMethod
    public void disconnect(PluginCall call) {
        String id = call.getString("id");
        implementation.disconnect(id);
        call.resolve();
    }
}